package com.supwisdom.dlpay.consume.service.impl

import com.supwisdom.dlpay.consume.PersonTransBuilder
import com.supwisdom.dlpay.consume.dao.AccountDao
import com.supwisdom.dlpay.consume.dao.UserdtlDao
import com.supwisdom.dlpay.consume.domain.TAccount
import com.supwisdom.dlpay.consume.domain.TUserdtl
import com.supwisdom.dlpay.consume.service.AccountUtilServcie
import com.supwisdom.dlpay.consume.service.PersonBalancePayService
import com.supwisdom.dlpay.exception.TransactionProcessException
import com.supwisdom.dlpay.framework.dao.ShopaccDao
import com.supwisdom.dlpay.framework.dao.SubjectDao
import com.supwisdom.dlpay.framework.domain.TShopacc
import com.supwisdom.dlpay.framework.domain.TSubject
import com.supwisdom.dlpay.framework.domain.TTranstype
import com.supwisdom.dlpay.framework.service.SystemUtilService
import com.supwisdom.dlpay.framework.util.TradeDict
import com.supwisdom.dlpay.framework.util.TradeErrorCode
import com.supwisdom.dlpay.framework.util.Tradetype
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service
import javax.persistence.EntityManager
import javax.persistence.PersistenceContext


@Service
class AccountUtilServcieImpl : AccountUtilServcie {
    @Autowired
    lateinit var accountDao: AccountDao

    @Autowired
    lateinit var shopaccDao: ShopaccDao

    @Autowired
    lateinit var subjectDao: SubjectDao


    override fun readAccountForUpdate(userid: String, nowait: Boolean): TAccount {
       //TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
        return accountDao.findByUserid(userid);
    }

    override fun readAccount(userid: String): TAccount {
        accountDao.findByUserid(userid).let {
            if (null != it) {
                return it
            }
            throw TransactionProcessException(TradeErrorCode.ACCOUNT_NOT_EXISTS, "账户<$userid>不存在")
        }
    }

    override fun readShopAccForUpdate(shopId: Int, nowait: Boolean): TShopacc {
       // TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
        return shopaccDao.findByShopid(shopId)
    }

    override fun readShopAcc(shopId: Int): TShopacc {
        shopaccDao.findByShopid(shopId).let {
            if (null != it) {
                return it
            }
            throw TransactionProcessException(TradeErrorCode.SHOP_NOT_EXISTS, "商户<$shopId>不存在")
        }
    }

    override fun readSubjectForUpdate(subjno: String, nowait: Boolean): TSubject {
        //TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
        return subjectDao.getOne(subjno);
    }

    override fun readSubject(subjno: String): TSubject {
        subjectDao.getOne(subjno).let {
            if (null != it) {
                return it
            }
            throw TransactionProcessException(TradeErrorCode.SUBJECT_NOT_EXISTS, "科目<$subjno>不存在")
        }
    }

    override fun readTranstype(transtype: Int): TTranstype {
        //TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
        return TTranstype();
    }
}

@Service
class PersonBalancePayServiceImpl : PersonBalancePayService {
    @Autowired
    lateinit var userdtlDao: UserdtlDao

    @Autowired
    lateinit var accountDao: AccountDao

    @PersistenceContext
    lateinit var em: EntityManager

    @Autowired
    lateinit var systemUtilService: SystemUtilService


    private fun lockAccount(builder: PersonTransBuilder): TAccount {
        TODO("")
    }

    override fun process(builder: PersonTransBuilder): TUserdtl {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun init(builder: PersonTransBuilder): TUserdtl {

        val userdtl = TUserdtl()
        val refno = systemUtilService.refno
        val systemtime = systemUtilService.sysdatetime
        userdtl.refno = refno
        userdtl.amount = builder.amount
        userdtl.createtime = systemtime.hostdatetime
        userdtl.status = TradeDict.DTL_STATUS_INIT
        userdtl.transdate = builder.transDate
        userdtl.transtime = builder.transTime
        userdtl.userid = builder.person?.userid
        userdtl.outtradeno = builder.outtradeno
        userdtl.payinfo = builder.payinfo
        userdtl.paytype = builder.paytype
        userdtl.tradeflag = 2
        userdtlDao.save(userdtl)
        return userdtl
    }

    override fun wip(paydtl: TUserdtl, builder: PersonTransBuilder): TUserdtl {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun wip(refno: String, builder: PersonTransBuilder): TUserdtl {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun finish(paydtl: TUserdtl, status: String, builder: PersonTransBuilder): TUserdtl {
        if(paydtl.status==TradeDict.DTL_STATUS_SUCCESS){
            return paydtl
        }
        paydtl.status = status
        if(status==TradeDict.DTL_STATUS_SUCCESS){
            //TODO 更新商户余额，科目余额，账户支付的需要更新余额
        }
        val systemtime = systemUtilService.sysdatetime
        paydtl.endtime = systemtime.hostdatetime
        userdtlDao.save(paydtl)
        return paydtl
    }

    override fun finish(refno: String, status: String, builder: PersonTransBuilder): TUserdtl {
        userdtlDao.findById(refno).let {
            if (it.isPresent) {
                return finish(it.get(), status, builder)
            }
            throw TransactionProcessException(TradeErrorCode.TRANSACTION_NOT_EXISTS, "交易参考号<$refno>不存在")
        }
    }


}